<?php
/**
*
* @package Moods
* @copyright (c) 2016 Theriddler - (http://www.peppersrus-forum.nl)
* @copyright (c) 2015 Rmcgirr83
* @Code lent from nationalflags extension by Rmcgirr83
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*/

namespace theriddler\moods\event;

/**
* @ignore
*/
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
* Event listener
*/
class listener implements EventSubscriberInterface
{
	/** @var \phpbb\auth\auth */
	protected $auth;

	/** @var \phpbb\cache\service */
	protected $cache;

	/** @var \phpbb\config\config */
	protected $config;

	/** @var \phpbb\controller\helper */
	protected $helper;

	/** @var \phpbb\db\driver\driver */
	protected $db;

	/** @var \phpbb\request\request */
	protected $request;

	/** @var \phpbb\template\template */
	protected $template;

	/** @var \phpbb\user */
	protected $user;

	/** @var \phpbb\extension\manager "Extension Manager" */
	protected $ext_manager;

	/** @var string phpBB root path */
	protected $phpbb_root_path;

	/** @var string phpEx */
	protected $php_ext;

	/* @var \theriddler\moods\core\moods */
	protected $functions;

	/**
	* Constructor
	*
	* @param \phpbb\auth\auth					$auth			Auth object
	* @param \phpbb\cache\service				$cache			Cache object
	* @param \phpbb\config\config               $config         Config object
	* @param \phpbb\controller\helper           $helper         Controller helper object
	* @param \phpbb\db\driver\driver			$db				Database object
	* @param \phpbb\request\request				$request		Request object
	* @param \phpbb\template\template           $template       Template object
	* @param \phpbb\user                        $user           User object
	* @param \phpbb\extension\manager			$ext_manager		Extension manager object
	* @param string                             $phpbb_root_path      phpBB root path
	* @param string                             $php_ext        phpEx
	* @param \theriddler\moods\core\moods		$functions	functions to be used by class
	* @access public
	*/
	public function __construct(
			\phpbb\auth\auth $auth,
			\phpbb\cache\service $cache,
			\phpbb\config\config $config,
			\phpbb\controller\helper $helper,
			\phpbb\db\driver\driver_interface $db,
			\phpbb\request\request $request,
			\phpbb\template\template $template,
			\phpbb\user $user,
			\phpbb\extension\manager $ext_manager,
			$phpbb_root_path,
			$php_ext,
			\theriddler\moods\core\moods $functions)
	{
		$this->auth = $auth;
		$this->cache = $cache;
		$this->config = $config;
		$this->helper = $helper;
		$this->db = $db;
		$this->request = $request;
		$this->template = $template;
		$this->user = $user;
		$this->ext_manager	 = $ext_manager;
		$this->root_path = $phpbb_root_path;
		$this->php_ext = $php_ext;
		$this->functions = $functions;

		$this->ext_path = $this->ext_manager->get_extension_path('theriddler/moods', true);
	}

	/**
	 * Assign functions defined in this class to event listeners in the core
	 *
	 * @return array
	 * @static
	 * @access public
	 */
	static public function getSubscribedEvents()
	{
		return array(
			'core.user_setup'							=> 'user_setup',
			'core.ucp_profile_modify_profile_info'		=> 'user_mood_profile',
			'core.ucp_profile_validate_profile_info'	=> 'user_mood_profile_validate',
			'core.ucp_profile_info_modify_sql_ary'		=> 'user_mood_profile_sql',
			'core.acp_users_modify_profile'				=> 'acp_user_mood_profile',
			'core.acp_users_profile_modify_sql_ary'		=> 'user_mood_profile_sql',
			'core.viewtopic_assign_template_vars_before'	=> 'viewtopic_template_vars_before',
			'core.viewtopic_cache_user_data'			=> 'viewtopic_cache_user_data',
			'core.viewtopic_cache_guest_data'			=> 'viewtopic_cache_guest_data',
			'core.viewtopic_modify_post_row'			=> 'viewtopic_modify_post_row',
			'core.memberlist_view_profile'				=> 'memberlist_view_profile',
			'core.ucp_pm_view_messsage'					=> 'ucp_pm_view_messsage',
		);
	}

	/**
	 * Set up the moods and add the lang vars
	 *
	 * @param object $event The event object
	 * @return null
	 * @access public
	 */
	public function user_setup($event)
	{
		if (empty($this->config['allow_moods']))
		{
			return;
		}
		// Need to ensure the moods are cached on page load
		$this->functions->cache_moods();
		$lang_set_ext = $event['lang_set_ext'];
		$lang_set_ext[] = array(
			'ext_name' => 'theriddler/moods',
			'lang_set' => 'common',
		);
		$event['lang_set_ext'] = $lang_set_ext;
	}

	/**
	 * Allow users to change their mood
	 *
	 * @param object $event The event object
	 * @return null
	 * @access public
	 */
	public function user_mood_profile($event)
	{
		if (empty($this->config['allow_moods']))
		{
			return;
		}

		// Request the user option vars and add them to the data array
		$event['data'] = array_merge($event['data'], array(
			'user_mood'	=> $this->request->variable('user_mood', (int) $this->user->data['user_mood']),
		));

		$this->display_mood_options($event);
	}

	/**
	 * Validate users changes to their mood
	 *
	 * @param object $event The event object
	 * @return null
	 * @access public
	 */
	public function user_mood_profile_validate($event)
	{
		if (empty($this->config['allow_moods']))
		{
			return;
		}

	}

	/**
	 * User changed their mood so update the database
	 *
	 * @param object $event The event object
	 * @return null
	 * @access public
	 */
	public function user_mood_profile_sql($event)
	{
		if (empty($this->config['allow_moods']))
		{
			return;
		}
		$event['sql_ary'] = array_merge($event['sql_ary'], array(
				'user_mood' => $event['data']['user_mood'],
		));
	}

	/**
	 * Allow admins to change user moods
	 *
	 * @param object $event The event object
	 * @return null
	 * @access public
	 */
	public function acp_user_mood_profile($event)
	{
		if (empty($this->config['allow_moods']))
		{
			return;
		}

		// Request the user option vars and add them to the data array
		$event['data'] = array_merge($event['data'], array(
			'user_mood'	=> $this->request->variable('user_mood', $event['user_row']['user_mood']),
		));

		$this->display_mood_options($event);
	}

	/**
	 * Load the css file
	 *
	 * @param object $event The event object
	 * @return null
	 * @access public
	 */
	public function viewtopic_template_vars_before($event)
	{
		if (empty($this->config['allow_moods']))
		{
			return;
		}

		$this->template->assign_vars(array(
			'S_MOODS'		=> true,
		));
	}
	/**
	 * Update viewtopic user data
	 *
	 * @param object $event The event object
	 * @return null
	 * @access public
	 */
	public function viewtopic_cache_user_data($event)
	{
		if (empty($this->config['allow_moods']))
		{
			return;
		}

		$array = $event['user_cache_data'];
		$array['user_mood'] = $event['row']['user_mood'];
		$event['user_cache_data'] = $array;
	}

	/**
	 * Update viewtopic guest data
	 *
	 * @param object $event The event object
	 * @return null
	 * @access public
	 */
	public function viewtopic_cache_guest_data($event)
	{
		if (empty($this->config['allow_moods']))
		{
			return;
		}

		$array = $event['user_cache_data'];
		$array['user_mood'] = 0;
		$event['user_cache_data'] = $array;
	}

	/**
	 * Modify the viewtopic post row
	 *
	 * @param object $event The event object
	 * @return null
	 * @access public
	 */
	public function viewtopic_modify_post_row($event)
	{
		if (empty($this->config['allow_moods']))
		{
			return;
		}

		// If setting in ACP is set to not allow guests and bots to view the moods
		if (empty($this->config['moods_display_to_guests']) && ($this->user->data['is_bot'] || $this->user->data['user_id'] == ANONYMOUS))
		{
			return;
		}

		$mood = $this->functions->get_user_mood($event['user_poster_data']['user_mood']);

		$event['post_row'] = array_merge($event['post_row'], array(
			'USER_MOOD' => $mood,
		));
	}

	/**
	 * Display mood on viewing user profile
	 *
	 * @param object $event The event object
	 * @return null
	 * @access public
	 */
	public function memberlist_view_profile($event)
	{
		if (empty($this->config['allow_moods']))
		{
			return;
		}

		if (!empty($event['member']['user_mood']))
		{
			$mood = $this->functions->get_user_mood($event['member']['user_mood']);

			$this->template->assign_vars(array(
				'USER_MOOD'		=> $mood,
				'S_MOODS'		=> true,
			));
		}
	}

	/**
	 * Display mood on viewing PM's
	 *
	 * @param object $event The event object
	 * @return null
	 * @access public
	 */
	public function ucp_pm_view_messsage($event)
	{
		if (empty($this->config['allow_moods']))
		{
			return;
		}

		if (!empty($event['user_info']['user_mood']))
		{
			$mood = $this->functions->get_user_mood($event['user_info']['user_mood']);

			$array = $event['msg_data'];
			$array['USER_MOOD'] = $mood;
			$event['msg_data'] = $array;

			$this->template->assign_vars(array(
				'S_MOODS'		=> true,
			));
		}
	}

	/**
	 * Display mood for user selection
	 *
	 * @param object $event The event object
	 * @return null
	 * @access private
	 */
	private function display_mood_options($event)
	{
		$moods = $this->get_mood_cache();
		$mood_name = $mood_image = '';
		$mood_id = 0;
		if ($event['data']['user_mood'])
		{
			$mood_name = $moods[$event['data']['user_mood']]['mood_name'];
			$mood_image = strtolower($moods[$event['data']['user_mood']]['mood_image']);
			$mood_id = $moods[$event['data']['user_mood']]['mood_id'];
		}

		$s_mood_options = $this->functions->list_moods($event['data']['user_mood']);

		$this->template->assign_vars(array(
			'USER_MOOD'		=> $event['data']['user_mood'],
			'MOOD_IMAGE'	=> ($mood_image) ? $this->ext_path . 'moods/' . $mood_image : '',
			'MOOD_NAME'		=> $mood_name,
			'S_MOOD_OPTIONS'	=> $s_mood_options,
			'S_MOODS'			=> true,
			'AJAX_MOOD_INFO' 	=> $this->helper->route('theriddler_moods_getmood', array('mood_id' => 'MOOD_ID')),
		));
	}

	/**
	 * Get the cache of the moods
	 *
	 * @return string mood_cache
	 * @access private
	 */
	private function get_mood_cache()
	{
		return $this->cache->get('_user_moods');
	}
}
